#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/xpm.h>
#include <X11/Shell.h>
#include <Xm/MenuShell.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <Xm/Xm.h>

#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/Display.h>


/* structure that is used to create widgets, store pixmaps and hold command
   lines */

int form_index, no_forms, screen, howmany;

Display *display;

/****************************************/
/* This structure will be used to       */
/* contain data to reparent windows. It */
/* will be filled in in the parser loop */
/****************************************/

typedef struct {
  char name[32];
  Widget form;
  Window absorbed;
  int shadow_w, x, y, w, h;
  int toolbar;
} absorb_struct;

/* The button structure */

typedef struct{
  Widget *widget;
  char cmd_line[255];        /* Commandline to execute, is done by */
  Pixmap pixmap;             /* means of the system command. Also  */
} button_struct;             /* used as name-of-pixmap storage.    */

absorb_struct global_absorb[8];    /* up to eight clients can be absorbed */

/* More than 8 absorb_structs are not neccesary, unless you want to absorb
   clients like netscape, xterm, xcalc, etc. etc. If you want more, just
   make it bigger ;) */

/*

Copyright (c) 1987  X Consortium

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall
not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization
from the X Consortium.

*/

Window Window_With_Name(Display *dpy, Window top, char *name) {
  Window *children, dummy;
  unsigned int nchildren;
  int i;
  Window w=0;
  char *window_name;

  if (XFetchName(dpy, top, &window_name) && !strcmp(window_name, name))
    return(top);

  if (!XQueryTree(dpy, top, &dummy, &dummy, &children, &nchildren))
    return(0);

  for (i=0; i<nchildren; i++) {
    w = Window_With_Name(dpy, children[i], name);
    if (w)
      break;
  }
  if (children) XFree ((char *)children);
  return(w);
}

/*--------------------------------------*/
/* The callback routines...             */
/*--------------------------------------*/


void button_callback ( Widget widget, button_struct *ptr_butt, caddr_t call_data ) {
  system(ptr_butt->cmd_line);    /* execute a command */
}

/* It's better to unmap than to unrealize! Unmapping a window does not unmap
   its subwindows, so clients like xclock and xload may continue getting
   their graphical data. XtUnrealize unmaps subwindows as well, and we don't
   want that to happen, do we... */

void unreal_callback ( Widget widget, Widget *wid_ptr, caddr_t call_data ) {
/*  XtUnrealizeWidget(*wid_ptr);*/
    XUnmapWindow( display, XtWindow(*wid_ptr) );
}

void colorchange_callback ( Widget widget, Widget *wid_ptr, caddr_t call_data ) {
  XtVaSetValues( *wid_ptr,
    XtVaTypedArg, XmNbackground, XmRString, "yellow2", 8,
    XtVaTypedArg, XmNarmColor, XmRString, "yellow2", 8,
    XtVaTypedArg, XmNtopShadowColor, XmRString, "yellow1", 8,
    XtVaTypedArg, XmNbottomShadowColor, XmRString, "yellow4", 8,
  NULL );
}

void real_callback ( Widget widget, Widget *wid_ptr, caddr_t call_data ) {
  XtVaSetValues( widget,
    XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
    XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
    XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
    XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
  NULL );
/*  XtRealizeWidget(*wid_ptr);*/
    XMapWindow( display, XtWindow(*wid_ptr) );
}

void realall_callback ( Widget widget, Widget *wid_ptr[23], caddr_t call_data ) {
  for(form_index=1; form_index<=no_forms; form_index++) {
/*    XtRealizeWidget( *wid_ptr[form_index] );*/
    XMapWindow( display, XtWindow(*wid_ptr[form_index]) );
  }
}

void unrealall_callback ( Widget widget, Widget *wid_ptr[23], caddr_t call_data ) {
  for(form_index=1; form_index<=no_forms; form_index++) {
/*    XtUnrealizeWidget( *wid_ptr[form_index] );*/
    XUnmapWindow( display, XtWindow(*wid_ptr[form_index]) );
  }
}

void colorchangeall_callback ( Widget widget, Widget *wid_ptr[23], caddr_t call_data ) {
  for(form_index=1; form_index<=no_forms; form_index++) {
    XtVaSetValues( *wid_ptr[form_index],
      XtVaTypedArg, XmNbackground, XmRString, "yellow2", 8,
      XtVaTypedArg, XmNarmColor, XmRString, "yellow2", 8,
      XtVaTypedArg, XmNtopShadowColor, XmRString, "yellow1", 8,
      XtVaTypedArg, XmNbottomShadowColor, XmRString, "yellow4", 8,
    NULL );
  }
}

void colorrestoreall_callback ( Widget widget, Widget *wid_ptr[23], caddr_t call_data ) {
  for(form_index=1; form_index<=no_forms; form_index++) {
    XtVaSetValues( *wid_ptr[form_index],
      XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
      XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
      XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
      XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
    NULL );
  }
}


void quit_callback ( Widget widget, Display *display, caddr_t call_data ) {
  XtCloseDisplay(display);
  exit(0); /* and get loads of zombie's ;) */
}

/*------------------------------------------------------*/
/*                        MAIN                          */
/*------------------------------------------------------*/


int main ( int argc, char *argv[] ) {

  Arg args[20]; int n=0;

  Widget        xmdisplay_widget;

  button_struct buttons[20*20];    /* 400 buttons max */
  Widget forms[20];                /* 20 toolbars max */
  Widget shells[20];
  Widget form_select_buttons[23];  /* had to add one extra, for some reason */

  char temp_buff[255];    /* temporary storage, used in parser */
  char shell_name[16];    /* huh!? I dunno what this is... See for yourself */
  char to_absorb[32];     /* Oh! It's temporary storage as well ;) */

/*  Display *display;*/   /* Made global, since callbacks use it a lot */

  XtAppContext appcontext;

  XpmAttributes pix_attrib;
  int status;

/* filestream shit */

  FILE *file_stream;

  /* temporary storage: */

  int max_p, x, y, w, h, shadwidth, shad_w, fw, fh,
      which_form, allow_unrealize;
  int offsx, offsy;  

  /* loop counters: */

  int pp, pos, lastpp, q;


  pix_attrib.valuemask = 0;
  lastpp=0;

  offsx=0; offsy=0;

/* beginning of the interesting stuff. initialize the butt bunch */
/* the dimensions of this parent widget are dummy. the child form */
/* will override the dimensions set by parent */

  XtToolkitInitialize();
  appcontext=XtCreateApplicationContext();
  display=XtOpenDisplay(appcontext, NULL, "parent", "mybuttons", NULL, 0, &argc, argv);

  screen=DefaultScreen(display);

/* disable drag 'n drop... very important, because if we don't, we're  */
/* bound to get segmentation faults and coredumps if we use the middle */
/* mouse button on anything sticking out or etched in or whatever      */

  xmdisplay_widget=XmGetXmDisplay(display);
  XtVaSetValues( xmdisplay_widget,
                 XmNdragInitiatorProtocolStyle, XmDRAG_NONE,
                 XmNdragReceiverProtocolStyle, XmDRAG_NONE,
  NULL );
 
  howmany=0;
  

/* read the datafile, if possible */

  if ( (file_stream = fopen(argv[1],"rb")) ) {

    fscanf(file_stream, "%s", to_absorb);    /* see if we have to absorb */

    if (strcmp(to_absorb,"absorb")==0) {
/*      for(q=1; q<=80000000; q++);*/ /* let's not use this ;) only for */
                                      /* internal testing! */

      fscanf(file_stream, "%i", &howmany);    /* get number of absorbs */
#ifdef DEBUG
      fprintf(stderr,"%i",howmany);
#endif

/* get absorb data, speaks for itself */

      for(q=1; q<=howmany; q++) {
        fscanf(file_stream, "%s", to_absorb);
        fscanf(file_stream, "%i", &which_form);
        fscanf(file_stream, "%i", &shadwidth);
        fscanf(file_stream, "%i", &x);
        fscanf(file_stream, "%i", &y);
        fscanf(file_stream, "%i", &w);
        fscanf(file_stream, "%i", &h);

/*        fprintf(stderr,"%s\n", to_absorb); */

        global_absorb[q].shadow_w=shadwidth;
        global_absorb[q].x=x;
        global_absorb[q].y=y;
        global_absorb[q].w=w;
        global_absorb[q].h=h;
        global_absorb[q].toolbar=which_form;

#ifdef DEBUG
        fprintf(stderr,"%i\n",global_absorb[q].shadow_w);
        fprintf(stderr,"%i\n",global_absorb[q].x);
        fprintf(stderr,"%i\n",global_absorb[q].y);
        fprintf(stderr,"%i\n",global_absorb[q].w);
        fprintf(stderr,"%i\n",global_absorb[q].h);
        fprintf(stderr,"%i\n",global_absorb[q].toolbar);
#endif

/* 'ere the fun starts! grab the window ID of the client! */

        global_absorb[q].absorbed=Window_With_Name(display,
                                                   RootWindow(display, screen),
                                                   to_absorb);

/*      fprintf(stderr,"%i",howmany);*/


      }

/* unmap all clients, otherwise reparenting won't work */

      for(q=1; q<=howmany; q++)
        XUnmapWindow(display, global_absorb[q].absorbed);

    }

/* first number: >0 -> give bars a button to realize/unrealize
                 =0 -> don't
   the number also specifies the height of the button
*/

    fscanf(file_stream,"%i", &allow_unrealize);

/* read the number of bars we want to make
   if equal to 1, override previous flag (allow realize/unrealize)
*/

    fscanf(file_stream,"%i", &no_forms);  /* get number of forms (toolbars) */

    if (no_forms==0) {
      fprintf(stderr,"Error in datafile!\n
                      Number of forms is 0 at this point.\n");
      exit(-1);
    }

/* You'll have to undestand X to follow the next stuff. If you do, you wont
   need any comment to the source. First build the control shell (or not). */

    if (allow_unrealize>0 && no_forms!=1) {
      shells[0]=XtAppCreateShell("main", "main", applicationShellWidgetClass, display, args, n);

      n=0;

      XtSetArg( args[n], XmNx, 0 ); n++;
      XtSetArg( args[n], XmNy, 0 ); n++;
      XtSetArg( args[n], XmNwidth, 32*(no_forms+2) ); n++;                               
      XtSetArg( args[n], XmNheight, 16 ); n++;
      XtSetArg( args[n], XmNmaxWidth, 32*(no_forms+2) ); n++;                            
      XtSetArg( args[n], XmNmaxHeight, 16 ); n++;
      XtSetArg( args[n], XmNminWidth, 32*(no_forms+2) ); n++;                            
      XtSetArg( args[n], XmNminHeight, 16 ); n++;

      forms[0]=XtCreateManagedWidget( "form", xmFormWidgetClass, shells[0], args, n );

      XtVaSetValues( forms[0],
		     XmNhighlightThickness, 0,
		     XmNborderWidth, 0,
		     XmNshadowThickness, 0,
                     XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
                     XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
                     XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
      NULL );

      n=0;

      form_select_buttons[0]=XtCreateManagedWidget( "", xmPushButtonWidgetClass, forms[0], args, n );

      XtVaSetValues( form_select_buttons[0],
		     XmNx, 0,
		     XmNy, 0,
		     XmNwidth, 16,
		     XmNheight, 16,
		     XmNhighlightThickness, 0,
		     XmNborderWidth, 0,
		     XmNshadowThickness, 2,
         XtVaTypedArg, XmNbackground, XmRString, "red2", 5,
         XtVaTypedArg, XmNarmColor, XmRString, "red2", 5,
         XtVaTypedArg, XmNtopShadowColor, XmRString, "red1", 5,
         XtVaTypedArg, XmNbottomShadowColor, XmRString, "red4", 5,
      NULL );

      form_select_buttons[22]=XtCreateManagedWidget( "", xmPushButtonWidgetClass, forms[0], args, n );

      XtVaSetValues( form_select_buttons[22],
		     XmNx, 32,
		     XmNy, 0,
		     XmNwidth, 16,
		     XmNheight, 16,
		     XmNhighlightThickness, 0,
		     XmNborderWidth, 0,
		     XmNshadowThickness, 2,
         XtVaTypedArg, XmNbackground, XmRString, "blue2", 6,
         XtVaTypedArg, XmNarmColor, XmRString, "blue2", 6,
         XtVaTypedArg, XmNtopShadowColor, XmRString, "blue1", 6,
         XtVaTypedArg, XmNbottomShadowColor, XmRString, "blue4", 6,
      NULL );

      form_select_buttons[21]=
        XtCreateManagedWidget( "", xmPushButtonWidgetClass,
                               forms[0], args, n );

      XtVaSetValues( form_select_buttons[21],
		     XmNx, 48,
		     XmNy, 0,
		     XmNwidth, 16,
		     XmNheight, 16,
		     XmNhighlightThickness, 0,
		     XmNborderWidth, 0,
		     XmNshadowThickness, 2,
         XtVaTypedArg, XmNbackground, XmRString, "green2", 7,
         XtVaTypedArg, XmNarmColor, XmRString, "green2", 7,
         XtVaTypedArg, XmNtopShadowColor, XmRString, "green1", 7,
         XtVaTypedArg, XmNbottomShadowColor, XmRString, "green4", 7,
      NULL );

      for(form_index=1; form_index<=no_forms; form_index++) {
        form_select_buttons[form_index]=
          XtCreateManagedWidget( "", xmPushButtonWidgetClass,
                                 forms[0], args, n );

        XtVaSetValues( form_select_buttons[form_index],
		       XmNx, 32*(form_index+1),
		       XmNy, 0,
		       XmNwidth, 32,
		       XmNheight, 16,
		       XmNhighlightThickness, 0,
		       XmNborderWidth, 0,
		       XmNshadowThickness, 2,
           XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
           XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
           XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
           XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
        NULL );
      }

    }

/* scan the buttons in the datafile: */

    for(form_index=1; form_index<=no_forms; form_index++) {
      n = 0;

      offsx=0; offsy=0;

/* read the name of the shell (is also the class name) */

      fscanf(file_stream,"%s", shell_name);

/* read the dimensions of the form and the shadow thickness */

      fscanf(file_stream,"%i %i %i %i %i %i", &x, &y, &w, &h, &shad_w, &max_p);

      if (allow_unrealize>0 && no_forms!=1)
        fscanf(file_stream, "%s", temp_buff);

/*      if (w<60) w=60; */

      fw=w;
      fh=h;
      
      /* make a shell */

      shells[form_index]=XtAppCreateShell(shell_name, shell_name,
                                          applicationShellWidgetClass,
                                          display, args, n);

#ifdef DEBUG
      fprintf(stderr,"*");
#endif

/* make a form with the specified dimensions */

#ifdef DEBUG
      fprintf(stderr,"%i",allow_unrealize);
#endif

      if (allow_unrealize>0 && no_forms!=1) {
        if (!strcmp(temp_buff,"bottom")) h+=allow_unrealize;
        if (!strcmp(temp_buff,"top")   ) {
          offsy+=allow_unrealize;
          h+=allow_unrealize;
        }
        if (!strcmp(temp_buff,"left")  ) {
          offsx+=allow_unrealize;
          w+=allow_unrealize;
        }
        if (!strcmp(temp_buff,"right") ) w+=allow_unrealize;
      }

      for(q=1; q<=howmany; q++) {
        if (form_index==global_absorb[q].toolbar) {
          global_absorb[q].x+=offsx;
          global_absorb[q].y+=offsy;
        }
      }

      XtSetArg( args[n], XmNx, x ); n++;
      XtSetArg( args[n], XmNy, y ); n++;
      XtSetArg( args[n], XmNwidth, w ); n++;                               
      XtSetArg( args[n], XmNheight, h ); n++;
      XtSetArg( args[n], XmNmaxWidth, w ); n++;                            
      XtSetArg( args[n], XmNmaxHeight, h ); n++;
      XtSetArg( args[n], XmNminWidth, w ); n++;                            
      XtSetArg( args[n], XmNminHeight, h ); n++;

      forms[form_index]=XtCreateManagedWidget( "form", xmFormWidgetClass, shells[form_index], args, n );

#ifdef DEBUG
      fprintf(stderr,"*");
#endif

      XtVaSetValues( forms[form_index],
		     XmNhighlightThickness, 0,
		     XmNborderWidth, 0,
		     XmNshadowThickness, 0,
                     XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
                     XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
                     XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
      NULL );

#ifdef DEBUG
      fprintf(stderr,"*");
#endif

      n=0;

      pix_attrib.closeness = 40000;
      pix_attrib.valuemask |= XpmCloseness;

/* now create the realize/unrealize button if neccesary */

      if (allow_unrealize>0 && no_forms!=1) {
        if (!strcmp(temp_buff,"bottom")) {
          (Widget)buttons[lastpp].widget=XtCreateManagedWidget( "", xmPushButtonWidgetClass, forms[form_index], args, n );

          XtVaSetValues( (Widget)buttons[lastpp].widget,
		         XmNx, 0,
		         XmNy, fh,
		         XmNwidth, fw,
		         XmNheight, allow_unrealize,
		         XmNhighlightThickness, 0,
		         XmNborderWidth, 0,
		         XmNshadowThickness, shad_w,
             XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
             XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
             XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
             XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
          NULL );
        }
        if (!strcmp(temp_buff,"top")   ) {
          (Widget)buttons[lastpp].widget=XtCreateManagedWidget( "", xmPushButtonWidgetClass, forms[form_index], args, n );

          XtVaSetValues( (Widget)buttons[lastpp].widget,
		         XmNx, 0,
		         XmNy, 0,
		         XmNwidth, fw,
		         XmNheight, allow_unrealize,
		         XmNhighlightThickness, 0,
		         XmNborderWidth, 0,
		         XmNshadowThickness, shad_w,
             XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
             XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
             XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
             XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
          NULL );
        }
        if (!strcmp(temp_buff,"left")  ) {
          (Widget)buttons[lastpp].widget=XtCreateManagedWidget( "", xmPushButtonWidgetClass, forms[form_index], args, n );

          XtVaSetValues( (Widget)buttons[lastpp].widget,
		         XmNx, 0,
		         XmNy, 0,
		         XmNwidth, allow_unrealize,
		         XmNheight, fh,
		         XmNhighlightThickness, 0,
		         XmNborderWidth, 0,
		         XmNshadowThickness, shad_w,
             XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
             XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
             XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
             XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
          NULL );
        }
        if (!strcmp(temp_buff,"right") ) {
          (Widget)buttons[lastpp].widget=XtCreateManagedWidget( "", xmPushButtonWidgetClass, forms[form_index], args, n );

          XtVaSetValues( (Widget)buttons[lastpp].widget,
		         XmNx, fw,
		         XmNy, 0,
		         XmNwidth, allow_unrealize,
		         XmNheight, fh,
		         XmNhighlightThickness, 0,
		         XmNborderWidth, 0,
		         XmNshadowThickness, shad_w,
             XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
             XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
             XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
             XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
          NULL );
        }


/* and add the proper callbacks */

        XtAddCallback( (Widget)buttons[lastpp].widget,
                       XmNactivateCallback,
                       (XtCallbackProc) colorchange_callback,
        (XtPointer) &form_select_buttons[form_index] );
        XtAddCallback( (Widget)buttons[lastpp].widget,
                       XmNactivateCallback,
                       (XtCallbackProc) unreal_callback,
        (XtPointer) &shells[form_index] );

      }

      for ( pp=lastpp+1; pp<=lastpp+max_p; pp++) {

/* read some sizes and positions, all in one line */

#ifdef DEBUG
        fprintf(stderr,"*");
#endif

        fscanf(file_stream,"%i %i %i %i %i", &which_form, &x, &y, &w, &h);

#ifdef DEBUG
        fprintf(stderr,"*");
#endif

/* read name o/t pixmap (put it in the command line string) */

        pos=0;

        fscanf(file_stream,"%s",buttons[pp].cmd_line);
        
#ifdef DEBUG
      fprintf(stderr,"-");
#endif

        status = XpmReadFileToPixmap( display,
                                      XRootWindowOfScreen(XtScreen(shells[1])),
                                      buttons[pp].cmd_line,
                                      &(buttons[pp].pixmap), NULL, 
        &pix_attrib );
        if (status != XpmSuccess) {
          exit(1);
          fprintf(stderr, "Pixmap not found! Check the datafile!\n");
        }

#ifdef DEBUG
      fprintf(stderr,"-");
#endif

/* create the button */

        (Widget)buttons[pp].widget=XtCreateManagedWidget( "button", xmPushButtonWidgetClass, forms[which_form], args, n );

        XtVaSetValues( (Widget)buttons[pp].widget,
		       XmNx, x+offsx,
		       XmNy, y+offsy,
		       XmNwidth, w,
		       XmNheight, h,
		       XmNhighlightThickness, 0,
		       XmNborderWidth, 0,
		       XmNshadowThickness, shad_w,
		       XmNlabelType, XmPIXMAP,
		       XmNlabelPixmap, buttons[pp].pixmap,
		       XmNlabelInsensitivePixmap, buttons[pp].pixmap,
           XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
           XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,
           XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
           XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
        NULL );

#ifdef DEBUG
      fprintf(stderr,"*");
#endif

/* read the command line associated with the pixmap and put it in cmd_line */
/* then add a callback with the buttons[pp] struct as call data            */
/* button_callback will parse everything correctly from this point on      */

        pos=-1;
        do {
	  pos++;
	  fscanf(file_stream,"%c",&temp_buff[pos]);
        } while ( temp_buff[pos] != '~' );
        temp_buff[pos]='\n';
        temp_buff[pos]='\0';

        strcpy( buttons[pp].cmd_line, temp_buff );

        XtAddCallback( (Widget)buttons[pp].widget,
                       XmNactivateCallback,
                       (XtCallbackProc) button_callback,
        (XtPointer) &buttons[pp] );

#ifdef DEBUG
      fprintf(stderr,"*");
#endif

      }

      lastpp=pp;    /* stupid workaround */

    }

    if (allow_unrealize>0 && no_forms!=1) {
      for(form_index=1; form_index<=no_forms; form_index++) {

        XtAddCallback( form_select_buttons[form_index],
                       XmNactivateCallback,
                       (XtCallbackProc) real_callback,
        (XtPointer) &shells[form_index] );

      }
      XtAddCallback( form_select_buttons[0],
                     XmNactivateCallback,
                     (XtCallbackProc) quit_callback,
      (XtPointer) display );

      XtAddCallback( form_select_buttons[21],
                     XmNactivateCallback,
                     (XtCallbackProc) realall_callback,
      (XtPointer) shells );
      XtAddCallback( form_select_buttons[21],
                     XmNactivateCallback,
                     (XtCallbackProc) colorrestoreall_callback,
      (XtPointer) form_select_buttons );

      XtAddCallback( form_select_buttons[22],
                     XmNactivateCallback,
                     (XtCallbackProc) unrealall_callback,
      (XtPointer) shells );
      XtAddCallback( form_select_buttons[22],
                     XmNactivateCallback,
                     (XtCallbackProc) colorchangeall_callback,
      (XtPointer) form_select_buttons );
    }


    fclose (file_stream);

  }

  else {
    fprintf(stderr,"Datafile not found!\n");
    exit(666);
  }

/*----------------------------------------*/
/* REALIZE & REPARENT                     */

/* realize the butt stuff and let the X
   Toolkit take over the rest of the shit */

  for(form_index=1; form_index<=no_forms; form_index++) {
    XtRealizeWidget( shells[form_index] );
  }
  if (allow_unrealize>0 && no_forms!=1) XtRealizeWidget( shells[0] );

  q=0;
  n=0;

/* The reparent loop! */

  if(howmany>0) {
    for(q=1; q<=howmany; q++) {

/* first make a form on which the absorbed client will reside */

    global_absorb[q].form=
      XtCreateManagedWidget( "", xmFormWidgetClass,
                             forms[global_absorb[q].toolbar], args, n);

    XtVaSetValues( global_absorb[q].form,
       XmNleftAttachment, XmATTACH_NONE,
       XmNtopAttachment, XmATTACH_NONE,
 		   XmNx, global_absorb[q].x,
		   XmNy, global_absorb[q].y,
		   XmNwidth, global_absorb[q].w,
		   XmNheight, global_absorb[q].h,
		   XmNhighlightThickness, 0,
		   XmNborderWidth, 0,
		   XmNshadowThickness, global_absorb[q].shadow_w,
       XtVaTypedArg, XmNbackground, XmRString, "gray75", 7,
/*       XtVaTypedArg, XmNarmColor, XmRString, "gray75", 7,*/
       XtVaTypedArg, XmNtopShadowColor, XmRString, "gray85", 7,
       XtVaTypedArg, XmNbottomShadowColor, XmRString, "gray45", 7,
    NULL );

#ifdef DEBUG
      fprintf(stderr,"%i",global_absorb[q].toolbar);
#endif

/* Yeah!! Let's reparent the external proggies' windows!! */

/* Resistance is futile, X clients are futile, they will all
   be assimilated ;) */

      XReparentWindow(display, global_absorb[q].absorbed,
                      XtWindow(forms[(global_absorb[q].toolbar)]),
                      global_absorb[q].x, global_absorb[q].y);

/* Give it some proper sizes and position */

      XMoveResizeWindow(display, global_absorb[q].absorbed,
                        global_absorb[q].x+global_absorb[q].shadow_w,
                        global_absorb[q].y+global_absorb[q].shadow_w, 
                        global_absorb[q].w-(2*global_absorb[q].shadow_w),
                        global_absorb[q].h-(2*global_absorb[q].shadow_w));
      XSetWindowBorderWidth(display, global_absorb[q].absorbed, 0);

/* Map the window, before it's too late ;) */

      XMapWindow(display, global_absorb[q].absorbed);

    }
  }

/* Go into the loop */

  XtAppMainLoop(appcontext);

  return (0);

}
